package CheckRelease2::Approval;

use strict;
use base qw( MT::Object MT::Revisable );

use CheckRelease2::ApprovalStatus qw(:all);

use Exporter 'import';
our @EXPORT_OK = qw( NOT_COMPATIBLE CHECKING REMAND APPROVED DISMISSAL );

__PACKAGE__->install_properties({
	column_defs => {
    id              => 'integer not null auto_increment',
    object_id       => 'integer not null',
    content_type_id => 'integer',
    object_class    => 'string(255)',
    rev_id          => 'smallint',
    rev_number      => 'integer',
    current_step => {
    	type => 'integer',
    	revisioned => 1
    },
    rev_current_step => {
      type => 'integer',
      revisioned => 1,
    },
    status => {
    	type => 'integer',
    	revisioned => 1,
    },
    rev_status => {
      type => 'integer',
      revisioned => 1,
    },
    comments => {
    	type => 'text',
    	revisioned => 1,
    },
  },
  index => {
  	object_id => 1,
  	rev_id => 1,
  	status => 1,
  },
  datasource => 'cr2_approval',
  primary_key => 'id',
  audit => 1,
  meta => 1,
});

sub is_rev {
  my $self = shift;
  my ($value) = @_;
  if ( defined $value ) {
    $self->{is_rev} = $value ? 1 : 0;
  }
  defined $self->{is_rev} && $self->{is_rev} ? 1 : 0;
}

sub set_status {
  my $self = shift;
  my ($value) = @_;
  if ( $self->is_rev ) {
    $self->rev_status( $value );
  } else {
    $self->status( $value );
  }
}

sub get_status {
  my $self = shift;
  if ( $self->is_rev ) {
    $self->rev_status;
  } else {
    $self->status;
  }
}

sub set_current_step {
  my $self = shift;
  my ($value) = @_;
  if ( $self->is_rev ) {
    $self->rev_current_step( $value );
  } else {
    $self->current_step( $value );
  }
}

sub get_current_step {
  my $self = shift;
  if ( $self->is_rev ) {
    $self->rev_current_step || 0;
  } else {
    $self->current_step || 0;
  }
}

1;