package CheckRelease2::CMS;

use strict;

#use File::Spec;
use MT;
use CGI;
use CheckRelease2::Plugin;
use CheckRelease2::CMS::Entry;
use CheckRelease2::Util qw(_t _log get_param_session plugin unable_plugin lookup_rule add_alt_tmpl_path_mtcloud);
use MT::Component;
use Data::Dumper;

sub on_init_request {
  my ( $cb, $app ) = @_;
  $app ||= MT->instance;
  my $mode = $app->mode;
  my $q = $app->param;
  return 1 if unable_plugin;
  &content_data_callbacks($app);
  my $plugin = plugin();
  if ( $mode eq 'view' && $q->param('_type') eq 'cr2_step' ) {
  	$app->{plugin_template_path} = File::Spec->catfile($plugin->envelope, 'tmpl', 'cms');
    add_alt_tmpl_path_mtcloud();
  }

  my $token = $app->param('cr_token') || return 1;
  my $salt = $app->param('cr_salt');
  defined($token) && length($token) == 32 || return 1;
  defined($salt) && ($salt =~ m!^\d+$! && $salt <= 1000000) || return 1;

  my $hash = get_param_session($token, $salt);

  unless ( $hash ) {
    $app->mode('dashboard');
    $app->param('__mode', 'dashboard');
    $app->param('cr_error', Encode::encode_utf8(_t('This URL is invalid or expired.')));
    return 1;
  }

  my %param = $app->param_hash;
  foreach my $key ( keys %$hash ) {
    next if defined $param{$key} || $key eq 'magic_token';
    my $value = $hash->{$key};
    next unless defined $value;
    $app->param($key, Encode::encode_utf8($value));
  }

  1;
}

sub on_pre_run {
  my ( $cb, $app ) = @_;
  my $mode = $app->mode;
  my $q = $app->param;
  return 1 if unable_plugin;
# Add permissions on editing contents revision
  my $user = $app->user;
  if ( $mode eq 'view' && $q->param('crr') && $q->param('id') && $q->param('content_type_id') ) {
    my $perm = $app->permissions || $user->permissions;
    my $ct = $app->model('content_type')->load( $q->param('content_type_id') );
    if ( ref $ct && $perm->has( 'create_content_data:' .$ct->unique_id ) ) {
      $app->param('cr2_permission_org', $perm->clone())
        unless ref $app->param('cr2_permission_org');
      $perm->set_these_permissions('publish_content_data:' .$ct->unique_id);  
      $app->permissions( $perm );
    }
  }
}

sub content_data_callbacks {
  my $app = MT->instance;
  my $plugin = plugin;
  my $q = $app->param;
  my $type = $q->param('_type') || '';
  my $blog_id = $q->param('blog_id');
  my $sub_type = $q->param('type') ? $q->param('type') : '';
  my %params = $app->param_hash;
  if ( !($type eq 'entry' || $type eq 'page') && !$sub_type ) {
    my $ds = $q->param('datasource');
    $type ||= $ds || '';
  }
  if ( $sub_type =~ m!content_data_(\d+)$! || $type =~ m!content_data_(\d+)$! ) {
    $type = $1;
  }
  my $rule = lookup_rule($blog_id, $type);
  return 1 unless $rule;
  my $pkg = '$CheckRelease2::CheckRelease2::';
  my @ct = MT->model('content_type')->load();
  foreach ( @ct ) {
    $app->add_callback('cms_post_save.content_data.content_data_' .$_->id, 10, plugin, $pkg .'CMS::Entry::_hdlr_cms_post_save_entry');
    $app->add_callback('cms_filtered_list_param.content_data.content_data_' .$_->id, 10, plugin, $pkg .'CMS::Entry::_hdlr_cms_filtered_list_param');
    my $cls = 'content_data.content_data_' .$_->id;
    my $props = $app->registry('list_properties', $cls) || {};
    $props->{cr_approval_status} = {
      base => '__virtual.string',
      label => sub { _t('Approval Status'); },
      sort => 0,
      html => $pkg .'ListProperty::_hdlr_entry_approval_status_html',
      condition => $pkg .'ListProperty::_hdlr_condition',
      order => 100,
      data_type => 'string',
      display => 'default',
    };
    $props->{cr_revision} = {
      base => '__virtual.string',
      label => sub { _t('Revision'); },
      sort => 0,
      html => $pkg .'ListProperty::_hdlr_entry_revision_html',
      condition => $pkg .'ListProperty::_hdlr_condition',
      order => 210,
      data_type => 'string',
      display => 'default',
    };
    $app->registry('list_properties', $cls, $props);
  }
}
1;