package CheckRelease2::CMS::Entry;

use strict;
use MT;
use MT::Util qw(format_ts archive_file_for);
use CheckRelease2::ApprovalStatus qw(:all);
use CheckRelease2::Approval;
use CheckRelease2::Util qw(plugin _debug _dump _t _log unable_plugin empty array_search do_redirect
                            lookup_approval lookup_rule lookup_step can_final_approval 
                            can_intermediate_approval can_manage_workflow send_workflow_emails 
                            sanitize get_config_value register_future_que load_latest_revision
                            serialize unserialize set_meta gather_cb_beacon);
use MT::EntryStatus qw(:all);
use Data::Dumper;
use MT::ContentType;

sub _hdlr_template_source_edit_entry {
  my ($eh, $app, $tmpl) = @_;
  return $$tmpl if unable_plugin;
  my $partial = plugin()->load_tmpl('cms/partial/new_revision.tmpl');
  return $$tmpl unless defined $partial;
  my $html = $partial->text;
  my $src = '<mt:if name="recovered_failed">[\s\r\n]+<mtapp:statusmsg';
  $$tmpl =~ s!($src)!$html$1!m;
  my $type = $app->param('_type');
  if ( $type eq 'content_data' ) {
    $src = '<mt:if name="use_revision">';
  } else {
    $src = '<input type="hidden" name="old_status" id="old_status" value="<mt:var name="status">" />';
  }
  $html = '<input type="hidden" name="r" value="<mt:var name="r" />"><input type="hidden" name="crr" value="<mt:var name="crr" />">'; 
  $$tmpl =~ s!($src)!$html$1!m;
  if ( $app->param('crr') ) {
    $html = '<input type="hidden" name="__mode" value="save';
    my $alt = '<input type="hidden" name="__mode" value="save_entry_rev" />';

    if ( $type eq 'entry' || $type eq 'page' ) {
      $html .= '_entry" />';
      $$tmpl =~ s!$html!$alt!m;

      $html = '\.val\(\'save_entry\'\)';
      $alt = "\.val('save_entry_rev')";
      $$tmpl =~ s!$html!$alt!mg;
    } elsif ( $type eq 'content_data' ) {
      $html .= '" />';
      $alt = '<input type="hidden" name="__mode" value="save_cd_rev" />';
      $$tmpl =~ s!$html!$alt!mg;

      $html = '\.val\(\'save\'\)';
      $alt = "\.val('save_cd_rev')";
      $$tmpl =~ s!$html!$alt!mg;
    } else {
      $alt = '';
    }
  }

  # Change permission for users only permitted create. But it is problem that they can publish, so force remove publish options.
  my $crr = $app->param('crr') || undef;
  my $r = $app->param('r') || undef;
  my $user = $app->user;
  my $blog_id = $app->param('blog_id');
  # load original permission. $app->permissions is changed for revision
  my $perms = $app->param('cr2_permission_org') || $user->permissions($blog_id);
  my $can_publish = 0;
  if ( $type eq 'content_data' ) {
    my $ct_id = $app->param('content_type_id');
    my $ct = $app->model('content_type')->load( $ct_id )
      or return $$tmpl;
    $can_publish = $user->can_do('publish_all_content_data')
      || $perms->can_do('publish_all_content_data')
      || $perms->can_do('edit_all_content_data_'.$ct->unique_id)
      || $perms->can_do('publish_all_content_data_' .$ct->unique_id)
      || $perms->can_do('publish_own_content_data_' .$ct->unique_id);
  } elsif ( $type eq 'page' ) {
    $can_publish = $perms->can_do('manage_pages');
  } else {
    $can_publish = $perms->can_do('publish_post');
  }
  if ( !$can_publish ) {
    $$tmpl =~ s!<option value="2"<mt:if name="status_publish"> selected="selected"</mt:if>.+?</option>!!ims;
    $$tmpl =~ s!<mt:if name="status_review">.+?<option value="3" selected="selected">.*?<__trans phrase="Unpublished (Review)">.+?</mt:if>!!ims;
    $$tmpl =~ s!<option value="4"<mt:if name="status_future"> selected="selected"</mt:if>>.*?</option>!!imsg;
    $$tmpl =~ s!<mt:if name="status_unpublish">.+?<option value="6" selected="selected">.+?</mt:if>!!ims;
  }
  $$tmpl;
}

sub _hdlr_template_param_edit_entry {
  my ( $cb, $app, $param, $tmpl ) = @_;
  return 1 if unable_plugin;
  my $blog_id = $app->param('blog_id');
  my $type = $app->param('_type');
  my $class = $type;
  return 1 unless defined $type;

  my $q = $app->param;
  # find applyed rule
  if ( $type eq 'content_data' ) {
    $type = $q->param('content_type_id');
  }
  my $rule = lookup_rule($blog_id, $type)
    or return 1;

  # find current approval status
  my $object_id = $app->param('id');
  my $crr = $q->param('crr') || 0;
  my $r = $q->param('r') || 0;
  my ($approval);
  if ( defined $object_id ) {
    $approval = lookup_approval($object_id, $type, $r ? $r : undef);
  }
  # find workflow
  my $step = $app->model('cr2_step')->load({ id => $rule->step_id });

  my $partial = plugin()->load_tmpl('cms/widget/approval_status.tmpl');
  my $widget = $tmpl->createElement('app:widget', {
    id => 'cr2-approval-status-widget',
    class => 'cr2-approval-widget',
    label => _t('Approval'),
  });
  $widget->innerHTML($partial->text);
  my $anchor = $tmpl->getElementById('entry-publishing-widget');
  $tmpl->insertBefore( $widget, $anchor );

  # check permissions, current step and approval status
  $param->{cr2_status} = defined($approval) ? $approval->get_status : 0;
  $param->{can_manage_workflow} = can_manage_workflow($app->user, $blog_id);
  $param->{can_final_approval} = can_final_approval($app->user, $blog_id);
  $param->{workflow_name} = $step->name;
  $param->{workflow_id} = $step->id;
  $param->{type} = $type;
  $param->{lock_publish} = lock_publish($app, $approval);

  # comemnt
  if ( defined $approval ) {
    my $comment = $app->model('cr2_comment')->load(
      { cr2_approval_id => $approval->id },
      { sort => 'created_on', direction => 'descend', limit => 1 }
    );
    if ( defined $comment ) {
      $param->{last_comment} = $comment->comment;
      $param->{comment_date} = $comment->created_on;
      my $commenter = $app->model('author')->load({ id => $comment->created_by });
      if ( defined $commenter ) {
        $param->{commenter_name} = $commenter->nickname ;
        $param->{commenter_userpic_url} = $commenter->userpic_url;
      }
    }
  }

  $param->{crr} = $crr;
  $param->{r} = $r;
  $param->{loaded_revision} = 0 if $crr;
  if ( $crr == -1 ) {
    $param->{new_revision} = 1;
  } elsif ( $crr ) {
    $param->{edit_revision} = 1;
  }
  setup_option_perms($param, $app->user, $step, $approval);

  if ( $crr ) {
    my $data_col = $type;
    if ( $class eq 'page' ) {
      $data_col = 'entry';
    } elsif ( $class eq 'content_data' ) {
      $data_col = 'cd';
    }
    # restore categories
    my $rev_obj = load_latest_revision($app, $type, $param->{id});
    if ( ref $rev_obj ) {
      my $unpacked = unserialize( $rev_obj->$data_col );
      @{ $param->{selected_category_loop} } = map { $_->[0] } @{ $$unpacked->{__rev_cats} }
        if defined $$unpacked->{__rev_cats};
    }
    # Change template params, but not permit to publish users who are not permitted publish
    my $user = $app->user;
    my $blog_id = $app->param('blog_id');
    my $perms = $app->permissions;
    my $can_publish = 0;
    if ( $type eq 'content_data' ) {
      my $ct_id = $app->param('content_type_id');
      my $ct = $app->model('content_type')->load( $ct_id )
        or return $$tmpl;
      $can_publish = $user->can_do('publish_all_content_data')
        || $perms->can_do('publish_all_content_data')
        || $perms->can_do('edit_all_content_data_'.$ct->unique_id)
        || $perms->can_do('publish_all_content_data_' .$ct->unique_id)
        || $perms->can_do('publish_own_content_data_' .$ct->unique_id);
    } elsif ( $type eq 'page' ) {
      $can_publish = $perms->can_do('manage_pages');
    } else {
      $can_publish = $perms->can_do('publish_post');
    }
    if ( get_config_value('lock_publish', 'blog:'.$blog_id) ) {
      my $approval = lookup_approval($app->param('id'), $class, $r);
      if ( ref $approval && $approval->rev_status != APPROVED() ) {
        $can_publish = 0;
      }
    }
    # force show select of publish status
    $param->{can_publish_post} = 1 unless $can_publish;
    $param->{status_publish} = 0;
  }

  1;
}

sub lock_publish {
  my ($app, $approval) = @_;
  my $blog_id = $app->param('blog_id') || 0;
  my $scope = $blog_id == 0 ? 'system' : 'blog:'.$blog_id;
  return 0 if can_final_approval($app->user);
  return 0 unless get_config_value('lock_publish', $scope);
  return 1 if !defined($approval);
  return 0 
    if defined($approval->get_status)
      && $approval->get_status == APPROVED;
  1;
}

sub setup_option_perms {
  my ($param, $user, $step, $approval) = @_;
  my $steps = $step->step_tree;
  my $step_num = @$steps;
  my ($current, $status) = (0, 0);
  if ( defined $approval ) {
    $status = $approval->get_status || 0;
    $current = !$status ? 0 : $approval->get_current_step;
  }
  # check base permission
  my $blog_id = $param->{blog_id} || 0;
  my $can_final = can_final_approval($user, $blog_id);
  my $can_intermediate = can_intermediate_approval($user, $blog_id);
  $param->{current_step} = $current + 1;
  $param->{max_step} = $step_num + 1;
  $param->{is_last_check} = $param->{current_step} + 1 == $param->{max_step} ? 1 : 0;

  # not compatible
  $param->{can_check} = 1 if !defined $approval || $status == 0 || @$steps > $current;
  if ( defined $approval && $current > 0 ) {
    my $author_ids = $steps->[ $current - 1 ];
    $param->{not_permitted} = 1 if $param->{id} && !( @$author_ids && array_search($author_ids, $user->id) >= 0 );
  }
  $param->{not_permitted} = 0 if $can_final;

  unless ( $param->{not_permitted} ) {
    if ( $status == CHECKING ) {
      if ( $current == $step_num ) {
        $param->{can_final_approval} = 1 if $can_final;
      }
    } elsif ( $status == REMAND ) {
      $param->{can_check} = 1;
    }
  }
  $param->{can_remand} = 1 if $current > 0 && ($can_intermediate || $can_final);
  $param->{can_dismissal} = 1 if $status > 0 && $can_final;

  $param->{is_approved} = $can_final;
  if ( get_config_value('lock_publish', 'blog:'.$blog_id) ) {
    my $get_status = defined($approval) && !empty($approval->get_status)
      ? $approval->get_status : -1;
    unless ( $get_status == APPROVED ) {
      $param->{is_approved} = 0;
      $param->{status_publish} = 0
    }
  } else {
    $param->{is_approved} = 1;
  }

}

sub _hdlr_template_source_header {
  my ( $cb, $app, $tmpl ) = @_;
  my $q = $app->param;
  my $mode = $q->param('__mode') || '';
  my $type = $q->param('_type') || '';
  # if recover auto_save data, __mode is save_xxxx
  if ( ($mode eq 'view' || $mode =~ m!^save_(entry|page|content_data)!) && $q->param('_type') =~ m!(entry|page|content_data)! ) {
    my $src = '<link rel="stylesheet" href="' .$app->static_path .'plugins/CheckRelease2/css/view.css">';
    $src .= '<script src="' .$app->static_path .'plugins/CheckRelease2/js/view.js"></script>';
    $$tmpl =~ s!(<mt:var name="html_head">)!$src$1!m;
  }
  if ( $mode eq 'list' && ($type eq 'entry' || $type eq 'page' || $type eq 'content_data') ) {
    my $src = '<link rel="stylesheet" href="' .$app->static_path .'plugins/CheckRelease2/css/common/list.css">';
    $$tmpl =~ s!(<mt:var name="html_head">)!$src$1!m;
  }
  $$tmpl;
}

sub _hdlr_cms_pre_save_entry {
  my ($cb, $app, $obj, $obj_orig) = @_;
  return 1 if unable_plugin;
  my $q = $app->param;
  my $blog_id = $q->param('blog_id');
  my $class = $q->param('_type');
  my $new_status = $q->param('cr2_status_new');

  # which step and who was posted?( move this pre_save_filter )
  my $author = $app->user;
  my $can_intermediate_approval = can_intermediate_approval($author, $blog_id);
  my $can_final_approval = can_final_approval($author, $blog_id);
  
  # TODO: validate simple permission and on workflow permission
  if ( $new_status == REMAND ) {
    return $app->error( _t('Can not change approval status [_1]. Permission denied.', _t('Remand')) )
      unless $can_intermediate_approval || $can_final_approval;
  } elsif ( $new_status == APPROVED ) {
    return $app->error( _t('Can not change approval status [_1]. Permission denied.', _t('Approve')) )
      unless $can_final_approval;
  }
  1;
}

sub save_rev {
  my $app = shift;
  my $q = $app->param;
  if ( $q->param('status') == MT::Entry::RELEASE() ) {
    my $class = $q->param('_type');
    my $id = $q->param('id');
    my $obj = $app->model($class)->load($id);
    $class = $obj->content_type_id if $class =~ m!^content_data!;
    my $rev = load_latest_revision($app, $class, $obj->id);
    $app->param('current_revision', $rev->rev_number);
    require MT::CMS::Entry;
    return MT::CMS::Entry::save($app, @_);
  } 
  $app->validate_magic or return;

  $app->remove_preview_file;
  if ( $app->param('is_power_edit') ) {
      return $app->save_entries(@_);
  }
  my $author = $app->user;
  my $type = $app->param('_type') || 'entry';

  my $class = $app->model($type)
      or return $app->errtrans("Invalid parameter");

  my $cat_class = $app->model( $class->container_type );
  my $perms = $app->permissions
      or return $app->permission_denied();
  return $app->errtrans('Invalid request.')
      if $app->param('class');
  my $id = $app->param('id');
  if ( !$id ) {
      if ( $type eq 'page' ) {
          return $app->permission_denied()
              unless $perms->can_do('create_new_page');
      }
      elsif ( $type eq 'entry' ) {
          return $app->permission_denied()
              unless $perms->can_do('create_new_entry');
      }
      else {
          return $app->errtrans("Invalid request.");
      }
  }
  else {
      if ( $type eq 'page' ) {
          return $app->permission_denied()
              unless $perms->can_do('edit_all_pages');
      }
  }
  # check for autosave
  if ( $app->param('_autosave') ) {
      return $app->autosave_object();
  }

  require MT::Blog;
  my $blog_id = $app->param('blog_id');
  my $blog    = MT::Blog->load($blog_id)
      or return $app->error(
      $app->translate( 'Cannot load blog #[_1].', $blog_id ) );

  my $archive_type;

  my ( $obj, $orig_obj, $orig_file );
  if ($id) {
      $obj = $class->load($id)
          || return $app->error(
          $app->translate( "No such [_1].", $class->class_label ) );
      return $app->error( $app->translate("Invalid parameter") )
          unless $obj->blog_id == $blog_id;
      if ( $type eq 'entry' ) {
          return $app->permission_denied()
              unless $perms->can_edit_entry( $obj, $author );
# comment out for not permitted publish users. Allow them to save a revision object.
#          return $app->permission_denied()
#              if ( $obj->status ne $app->param('status') )
#              && !( $perms->can_edit_entry( $obj, $author, 1 ) );
          $archive_type = 'Individual';
      }
      elsif ( $type eq 'page' ) {
          $archive_type = 'Page';
      }
      else {
          return $app->errtrans("Invalid request.");
      }
      $orig_obj = $obj->clone;
      $orig_file = archive_file_for( $orig_obj, $blog, $archive_type );
  }
  else {
      $obj      = $class->new;
      $orig_obj = $obj->clone;
  }
  my ( $primary_category_old, $categories_old );
  if ( $orig_obj->id ) {
      $primary_category_old = $orig_obj->category;
      $categories_old       = $orig_obj->categories;
  }
  my $status_old = $id ? $obj->status : 0;
  my $names = $obj->column_names;

  ## Get rid of category_id param, because we don't want to just set it
  ## in the Entry record; save it for later when we will set the Placement.
  my ( $cat_id, @add_cat ) = split /\s*,\s*/,
      ( $app->param('category_ids') || '' );
  $app->delete_param('category_id');
  if ($id) {
      ## Delete the author_id param (if present), because we don't want to
      ## change the existing author.
      $app->delete_param('author_id');
  }
  my %values = map { $_ => scalar $app->param($_) } @$names;
  delete $values{'id'} unless $app->param('id');
  ## Strip linefeed characters.
  for my $col (qw( text excerpt text_more keywords )) {
      $values{$col} =~ tr/\r//d if $values{$col};
  }
  $values{allow_comments} = 0
      if !defined( $values{allow_comments} )
      || $app->param('allow_comments') eq '';
  delete $values{week_number}
      if ( $app->param('week_number') || '' ) eq '';
  delete $values{basename}
      unless $perms->can_do("edit_${type}_basename");
  require MT::Entry;
  $values{status} = MT::Entry::FUTURE() if $app->param('scheduled');
  $obj->set_values( \%values );
  $obj->allow_pings(0)
      if !defined $app->param('allow_pings')
      || $app->param('allow_pings') eq '';
  my $ao_d = $app->param('authored_on_date');
  my $ao_t = $app->param('authored_on_time');
  my $uo_d = $app->param('unpublished_on_date');
  my $uo_t = $app->param('unpublished_on_time');

  if ( !$id ) {

      #  basename check for this new entry...
      if (   ( my $basename = $app->param('basename') )
          && !$app->param('basename_manual')
          && $type eq 'entry' )
      {
          my $exist = $class->exist(
              { blog_id => $blog_id, basename => $basename } );
          if ($exist) {
              $obj->basename( MT::Util::make_unique_basename($obj) );
          }
      }
  }
  # Cut a basename by blog settings
  if ( $app->param('basename_manual') ) {
      first_n_words( $obj->basename, $blog->basename_limit );
  }

  if ( $type eq 'page' ) {

      # -1 is a special id for identifying the 'root' folder
      $cat_id = 0 if $cat_id == -1;
      my $dup_it = $class->load_iter(
          {   blog_id  => $blog_id,
              basename => $obj->basename,
              class    => 'page',
              ( $id ? ( id => $id ) : () )
          },
          { ( $id ? ( not => { id => 1 } ) : () ) }
      );
      my $folder;
      $folder = MT::Folder->load($cat_id) if $cat_id;
      while ( my $p = $dup_it->() ) {
          my ( $dup_path, $org_path );

          if ( $app->config('BasenameCheckCompat') ) {
              my $p_folder = $p->folder;
              $dup_path
                  = defined $p_folder ? $p_folder->publish_path() : '';
              $org_path = defined $folder ? $folder->publish_path() : '';
          }
          else {
              $dup_path = $p->permalink;

              my $url = $blog->site_url || "";
              $url .= '/' unless $url =~ m!/$!;
              $org_path
                  = $url . archive_file_for( $obj, $blog, 'Page', $folder );
          }

          return $app->error(
              $app->translate(
                  "This basename has already been used. You should use an unique basename."
              )
          ) if ( $dup_path eq $org_path );
      }

  }
  if ( $type eq 'entry' ) {
      $obj->status( MT::Entry::HOLD() )
          if !$id
          && !$perms->can_do('publish_own_entry')
          && !$perms->can_do('publish_all_entry');
  }

  my $filter_result
      = $app->run_callbacks( 'cms_save_filter.' . $type, $app );

  if ( !$filter_result ) {
      my %param = ();
      $param{error}       = $app->errstr;
      $param{return_args} = $app->param('return_args');
      $app->param( 'reedit', 1 );
      return $app->forward( "view", \%param );
  }

  # check to make sure blog has site url and path defined.
  # otherwise, we can't publish a released entry
  if ( ( $obj->status || 0 ) != MT::Entry::HOLD() ) {
      if ( !$blog->site_path || !$blog->site_url ) {
          return $app->error(
              $app->translate(
                  "Your blog has not been configured with a site path and URL. You cannot publish entries until these are defined."
              )
          );
      }
  }
  my ( $previous_old, $next_old );
  if ( $perms->can_do("edit_${type}_authored_on") && ($ao_d) ) {
      my %param = ();
      my $ao    = $ao_d . ' ' . $ao_t;
      my $ts    = MT::Util::valid_date_time2ts($ao);
      if ( !$ts ) {
          $param{error} = $app->translate(
              "Invalid date '[_1]'; 'Published on' dates must be in the format YYYY-MM-DD HH:MM:SS.",
              $ao
          );
      }
      $param{return_args} = $app->param('return_args');
      return $app->forward( "view", \%param ) if $param{error};
      if ( $obj->authored_on ) {
          $previous_old = $obj->previous(1);
          $next_old     = $obj->next(1);
      }
      $obj->authored_on($ts);
  }
  if (   $perms->can_do("edit_${type}_unpublished_on")
      && $obj->status != MT::Entry::UNPUBLISH() )
  {
      if ( $uo_d || $uo_t ) {
          my %param = ();
          my $uo    = $uo_d . ' ' . $uo_t;
          my $ts    = MT::Util::valid_date_time2ts($uo);
          if ( !$ts ) {
              $param{error} = $app->translate(
                  "Invalid date '[_1]'; 'Unpublished on' dates must be in the format YYYY-MM-DD HH:MM:SS.",
                  $uo
              );
          }
          require MT::DateTime;
          unless ( $param{error} ) {
              $param{error} = $app->translate(
                  "Invalid date '[_1]'; 'Unpublished on' dates should be dates in the future.",
                  $uo
                  )
                  if (
                  MT::DateTime->compare(
                      blog => $blog,
                      a    => { value => time(), type => 'epoch' },
                      b    => $ts
                  ) > 0
                  );
          }
          if ( !$param{error} && $obj->authored_on ) {
              $param{error} = $app->translate(
                  "Invalid date '[_1]'; 'Unpublished on' dates should be later than the corresponding 'Published on' date.",
                  $uo
                  )
                  if (
                  MT::DateTime->compare(
                      blog => $blog,
                      a    => $obj->authored_on,
                      b    => $ts
                  ) > 0
                  );
          }
          $param{show_input_unpublished_on} = 1 if $param{error};
          $param{return_args} = $app->param('return_args');
          return $app->forward( "view", \%param ) if $param{error};
          if ( $obj->unpublished_on ) {
              $previous_old = $obj->previous(1);
              $next_old     = $obj->next(1);
          }
          $obj->unpublished_on($ts);
      }
      else {
          $obj->unpublished_on(undef);
      }
  }
  my $is_new = $obj->id ? 0 : 1;

  MT::Util::translate_naughty_words($obj);

  my $mode_org = $app->mode;
  $app->mode('save_entry');
  $app->run_callbacks( 'cms_pre_save.' . $type, $app, $obj, $orig_obj )
      || return $app->error(
      $app->translate(
          "Saving [_1] failed: [_2]",
          $class->class_label, $app->errstr
      )
      );
  $app->mode($mode_org);
}

sub _hdlr_cms_post_save_entry {
  my ($cb, $app, $obj, $obj_orig) = @_;
  update_workflow(@_);
}

sub update_workflow {
  my ($cb, $app, $obj, $obj_orig) = @_;
  return 1 if unable_plugin;
  my $q = $app->param;
  my $blog_id = $q->param('blog_id');
  my $class = $q->param('_type');
  if ( !($class eq 'entry' || $class eq 'page') ) {
    $class = $q->param('content_type_id');
  }
  my $rule = lookup_rule($blog_id, $class)
    or return 1;
  # lookup current status
  my $rev_number = $q->param('r');
  my $approval = defined $rev_number && $rev_number > 0
    ? lookup_approval($obj->id, $class, $rev_number)
    : lookup_approval($obj->id, $class);
  $approval ||= MT->model('cr2_approval')->new;

  my $status = $approval->get_status;
  my $current = $approval->get_current_step || 0;
  my $step = lookup_step($rule->step_id)
    or return $app->error( _t('A step(ID: [_1]) not found.', $rule->step_id) ); # not found return on pre_save_filter ?
  my $steps = $step->step_tree;
  my $max = @$steps;
  # status is changed or not and return
  my %params = $app->param_hash;
  my $new_status = $q->param('cr2_status_new');
  if ( defined $new_status ) {
    if ( $new_status == 0 ) {
      # nothing to do
    } elsif ( $new_status == REMAND ) {
      --$current;
    } elsif ( $new_status == APPROVED ) {
      $current = $max;
    } elsif ( $new_status == DISMISSAL ) {
      $current = 0;
      $new_status = 2;
    } else {
      ++$current;
    }
    $current = $max if $current > $max;
    unless ( defined $approval->id ) {
      $approval->object_id( $obj->id );
      $approval->current_revision(1);
      if ( $class =~ m!^\d+$! ) {
        $approval->content_type_id($class)
      }
      $approval->object_class( $obj->class );
    }
    my $is_rev = $app->mode =~ m!save_.+_rev! ? 1 : 0;
    my $rev_status = $approval->rev_status;
    my $rev_current_step = $approval->rev_current_step;
    $approval->is_rev(1) if $is_rev;
    $approval->set_status( $new_status ) if $new_status;
    $approval->set_current_step( $current );
    if ( $obj->status == MT::EntryStatus::RELEASE() && $is_rev ) {
      $approval->status( $rev_status );
      $approval->current_step ( $rev_current_step );
      $approval->set_status( 0 );
      $approval->set_current_step( 0 );
    }
    $approval->current_revision( $approval->current_revision() + 1 );
    $approval->rev_number( defined($rev_number) ? $rev_number+1 : undef );
    unless ( $approval->save() ) {

      my $errstr = _t('Failure to save approval. [_1]', $approval->errstr);
      MT::log({
        blog_id => $blog_id,
        level => MT::Log::ERROR(),
        message => $errstr,
      });
      return $app->error( $errstr );
    }    
  }

  my $author = $app->user;
  my $comment = MT->model('cr2_comment')->new;
  $comment->cr2_approval_id( $approval->id );
  $comment->status( $new_status ) if defined $new_status && $new_status =~ m!^\d+$!;
  $comment->comment( sanitize($q->param('cr2_comment')) ) ;
  $comment->created_by( $author->id );
  unless ( $comment->save() ) {
    my $errstr = _t('Failure to save a comment. [_1]', $comment->errstr);
    MT::log({
      blog_id => $blog_id,
      level => MT::Log::ERROR(),
      message => $errstr,
    });
  }
  if ( (defined $new_status && $new_status =~ m!^\d+$!) || $q->param('cr2_comment') ) {
    send_workflow_emails($author, $app->blog, $class, $obj, $new_status, $steps, $current, $approval, $comment);
  }
}

sub _hdlr_app_pre_listing_list_revision {
  my ($cb, $app, $terms, $args, $param, $hasher_ref) = @_;
  $terms->{rev_number} = { '>' => 0 };
}

sub _hdlr_cms_pre_save_entry_rev {
  my ($cb, $app, $obj, $obj_orig) = @_;
  return 1 if unable_plugin;
  my $q = $app->param;
  my $crr = $q->param('crr') || 0;
  my %params = $app->param_hash;
  return 1 unless $crr;
  my $blog_id = $q->param('blog_id');
  my $type = $q->param('_type');
  my $class = $type;
  my $data = {};
  if ( !($type eq 'entry' || $type eq 'page') ) {
    $type = $q->param('content_type_id');
    my $block_editor_data = $app->param('block_editor_data');
    $obj->block_editor_data($block_editor_data);
  }
  my $r = $obj->current_revision;

  # return if status is publish
  my $approval = lookup_approval($obj->id, $type, defined($r) ? $r : undef);
  $approval ||= MT->model('cr2_approval')->new;
  my $status = $q->param('status');
  if ( $status == MT::Entry::RELEASE() ) {
    return 1;
  }
  gather_cb_beacon($app);

  my $rev_num = $crr == -1 ? 1 : $r;
  my $rule = lookup_rule($blog_id, $type)
    or return 1;
  my $current = $approval->get_current_step || 0;
  my $user = $app->user;
  my $packed = $obj->pack_revision();
  my @packed_obj_assets;
  my $__rev_cats = [];
  my $__rev_tags = [];
  if ( $class eq 'entry' || $class eq 'page' ) {
    my $model = $app->model($type)
      or return $cb->error( $app->errtrans('Invalid parameter') );
    my $cat_class = $app->model( $model->container_type );
    my $objass_class = $app->model('objectasset');
    my @objassets;
    my @asset_ids = grep { m!^\d+$! } split /,/, $q->param('include_asset_ids');
    my $iter = $objass_class->load_iter({ object_ds => $class, object_id => $obj->id });
    my %obj_assets;
    while ( my $obj_asset = $iter->() ) {
      $obj_assets{ $obj_asset->asset_id } = $obj_asset;
    } 
    foreach my $asset_id ( @asset_ids ) {
      unless ( defined $obj_assets{ $asset_id } ) {
        my $obj_asset = $objass_class->new;
        $obj_asset->blog_id( $blog_id );
        $obj_asset->asset_id( $asset_id );
        $obj_asset->object_ds( $class );
        $obj_asset->object_id( $obj->id );
        my $pack = pack_revision($obj_asset);
        push @packed_obj_assets, $pack;
      }
    }
    my ( @add_cats ) = split /\s*,\s*/,
      ( $q->param('category_ids') || '' );
    my $cat_type = $class eq 'entry' ? 'category' : 'folder';
    my %cats = map { $_->id => $_ } $app->model($cat_type)->load({ id => \@add_cats });
    my $cntr = 0;
    for my $cat_id ( @add_cats ) {
      next unless defined $cats{$cat_id};
      next unless $cats{$cat_id}->blog_id == $blog_id;
      push @$__rev_cats, [$cat_id, $cntr == 0 ? 1 : 0];
      ++$cntr;
    }
    require MT::Tag;
    my $tags = $q->param('tags');
    my $tag_delim = chr( $app->user->entry_prefs->{tag_delim} );
    my @tags = MT::Tag->split( $tag_delim, $tags );
    foreach my $tag_name (@tags) {
      my $tag = MT::Tag->load( { name => $tag_name },
         { binary => { name => 1 } } );
      unless ( $tag ) {
        $tag = new MT::Tag;
        $tag->name($tag_name);
        $tag->save or next;
      }
      push @$__rev_tags, $tag->id;
    }
    set_meta($app, $obj);
  }
  my $ds = $obj->datasource;
  my $rev_obj = load_latest_revision($app, $type, $obj->id);
  $rev_num = defined $rev_obj ? $rev_obj->rev_number : $obj->current_revision;
  my $obj_id = $ds eq 'content_data' ? 'cd_id' : $ds .'_id';
  $obj->gather_changed_cols($obj_orig, $app);
  local $obj->{column_values}->{current_revision} = $rev_num || 0;
  my $revision_note = $app->param('revision-note');
  $r = $obj->save_revision( $revision_note );
  $rev_obj = load_latest_revision($app, $type, $obj->id);

  if ( $class eq 'entry' || $class eq 'page' ) {
    $packed->{status} = $status;
    $packed->{created_by} = $user->id if $rev_num == 1;
    $packed->{objectasset} = \@packed_obj_assets;
    $packed->{__rev_cats} = $__rev_cats;
    $packed->{__rev_tags} = $__rev_tags;
    $rev_obj->$obj_id( $obj->id );
    $rev_obj->$ds( serialize(\$packed) );
  } else {
    # ContentData revision is crashed with by setting $obj->block_editor_data.
    # So re-save data column.
    my $data = $obj->data;
    my $rev_data = unserialize( $rev_obj->cd );
    $$rev_data->{data} = $data;
    $rev_obj->cd( serialize( $rev_data ) );
  }
  $rev_obj->save()
    or return $cb->error( $rev_obj->errstr );
  register_future_que($rev_obj, $ds)
    if $status == FUTURE;
  my $sess_obj = $app->autosave_session_obj;
  $sess_obj->remove if $sess_obj;
  update_workflow(@_);
  my $args = {
    r => $rev_obj->rev_number,
    crr => 1
  };
  foreach ( qw/blog_id id _type content_type_id/ ) {
    $args->{$_} = $q->param($_) if defined $q->param($_);
  }
  $args->{saved_changes} = 1;
  my $url = $app->mt_uri(
    mode => 'view',
    args => $args );
  return do_redirect($app, $url);
}

sub pack_revision {
  my $obj = shift;
  my $class = ref $obj || $obj;
  my $values = $obj->column_values;
  my $meta_values = $obj->meta;
  foreach my $key (keys %$meta_values) {
    next if $key eq 'current_version';
    $values->{$key} = $meta_values->{$key};
  }
  return $values;
}

sub _hdlr_cms_filtered_list_param {
  my ($cb, $app, $res, $objs) = @_;
  return if unable_plugin();
  my $user = $app->user;
  my $blog_id = $app->param('blog_id');
  return unless get_config_value('lock_publish', 'blog:'.$blog_id);
  return if can_final_approval($user, $blog_id);
  my $type = $app->param('datasource') || '';
  if ( $type =~ m!content_data_(\d+)$! ) {
    $type = $1;
  }
  foreach my $obj ( @{ $res->{objects} } ) {
    my $approval = lookup_approval($obj->[0], $type);
    if ( !defined $approval ) {
      $obj->[0] = '';
    } else {
      $obj->[0] = ''
        if $approval->get_status != APPROVED;
    }
  }
}
1;