package CheckRelease2::Template::Tags;

use strict;
use MT;
use CheckRelease2::Util qw(_t plugin lookup_approval);
use CheckRelease2::ApprovalStatus qw(:all);

sub _hdlr_has_cr2_comments {
  my ( $ctx, $args, $cond ) = @_;
  my $obj = $ctx->stash('entry') || $ctx->stash('content');
  return 0 unless defined $obj;
  my $approval = lookup_approval($obj->id, $obj->class_type);
  return 0 unless $approval;
  1;
}

sub _hdlr_comments {
  my ( $ctx, $args, $cond ) = @_;
  my $obj = $ctx->stash('entry') || $ctx->stash('content')
    or return $ctx->_no_entry_error();
  $ctx->stash('blog', $obj->blog);
  #[TODO] Bug fix
  my $terms = { entry_id => $obj->id };
  my $limit = defined $args->{limit} ? $args->{limit} : 1;
  my $offset = defined $args->{offset} ? $args->{offset} : 0;
  my $sort = defined $args->{sort_by} ? $args->{sort_by} : 'created_on';
  my $order = defined $args->{sort_order} ? $args->{sort_order} : 'descend';
  my $res     = '';
  my $tok     = $ctx->stash('tokens');
  my $builder = $ctx->stash('builder');
  my @comments = MT->model('cr2_comment')->load($terms,
  	{ limit => $limit,
  		offset => $offset,
  		sort => $sort,
  		direction => $order,
  	}
  );
  my @author_ids = map { $_->created_by } @comments;
  my %authors = map { $_->id => $_ } MT->model('author')->load({ id => \@author_ids });
  my @approval_ids = map { $_->cr2_approval_id } @comments;
  my %approvals = map { $_->id => $_ } MT->model('cr2_approval')->load({ id => \@approval_ids });
  my $i = 0;
  my $vars = $ctx->{__stash}{vars} ||= {};
  for my $c ( @comments ) {
    local $vars->{__first__}    = !$i;
    local $vars->{__last__}     = !defined $comments[ $i + 1 ];
    local $vars->{__odd__}      = ( $i % 2 ) == 0;            # 0-based $i
    local $vars->{__even__}     = ( $i % 2 ) == 1;
    local $vars->{__counter__}  = $i + 1;
    local $ctx->{__stash}{cr2_comment} = $c;
    my $author_id = $c->created_by;
    local $ctx->{__stash}{author} = $authors{$author_id}
      if defined $authors{$author_id};
    local $ctx->{__stash}{approval} = $approvals{ $c->cr2_approval_id }
      if defined $approvals{ $c->cr2_approval_id };
    my $out = $builder->build($ctx, $tok, $cond)
    	or return $ctx->error( $builder->errstr );
    $res .= $out;
  }
  $res;
}

sub _hdlr_comment_date {
  my ( $ctx, $args ) = @_;
  my $c = $ctx->stash('cr2_comment')
    or return '';
  $args->{ts} = $c->created_on;
  my $ts = $ctx->build_date($args);
  unless ( defined $ts ) {
    MT::log({
      blog_id => defined $ctx->stash('blog') ? $ctx->stash('blog')->kd : 0,
      level => MT::Log::ERROR,
      message => _t('Failure to build_date. [_1]', $ctx->errstr),
    });
  }
  $ts;
}

sub _hdlr_comment_text {
  my ( $ctx, $args ) = @_;
  my $c = $ctx->stash('cr2_comment')
    or return '';
  $c->comment;
}

sub _hdlr_approval_status_label {
  my ( $ctx, $args ) = @_;
  my $status = defined $ctx->stash('cr2_approval')
    ? $ctx->stash('cr2_approval')->status
    : undef;
  my $label = '';
  if ( $status == NOT_COMPATIBLE ) {
    $label = 'Not Compatible';
  } elsif ( $status == CHECKING ) {
    $label = 'Checking';
  } elsif ( $status == REMAND ) {
    $label = 'On Remand';
  } elsif ( $status == APPROVED ) {
    $label = 'Approved';
  } elsif ( $status == DISMISSAL ) {
    $label = 'Dismissal';
  } else {
    $label = 'Not Compatible';
  }
  _t($label);
}

sub _hdlr_comment_status {
  my ( $ctx, $args ) = @_;
  defined $ctx->stash('cr2_comment')
      ? $ctx->stash('cr2_comment')->status
      : undef;
}

sub _hdlr_comment_status_label {
  my ( $ctx, $args ) = @_;
  my $status = defined $ctx->stash('cr2_comment')
      ? $ctx->stash('cr2_comment')->status
      : undef;
  my $label = '';
  if ( $status == NOT_COMPATIBLE ) {
    $label = 'Not Compatible';
  } elsif ( $status == CHECKING ) {
    $label = 'Checking';
  } elsif ( $status == REMAND ) {
    $label = 'On Remand';
  } elsif ( $status == APPROVED ) {
    $label = 'Approved';
  } elsif ( $status == DISMISSAL ) {
    $label = 'Dismissal';
  } else {
    $label = 'Not Compatible';
  }
  _t($label);
}

sub _hdlr_has_cr2_status_changed {
  my ( $ctx, $args, $cond ) = @_;
  my $obj = $ctx->stash('entry') || $ctx->stash('content')
    or return $ctx->_no_entry_error();
  my $approval = lookup_approval($obj->id, $obj->class);
  return 0 unless defined $approval;
  my $comment = $ctx->stash('cr2_comment')
    or return $ctx->error( _t('Use [_1] in CheckRelease comment context.', 'HasCR2StatusChanged') );
  my @comments = MT->model('cr2_comment')->load({
        cr2_approval_id => $approval->id
      }, {
        sort => 'id',
        direction => 'descend',
        limit => 2,
      });
  my $prev;
  foreach ( @comments ) {
    $prev = $_ if $_->id != $comment->id;
  }
  return 1 if !defined $prev && $comment->status;
  return 0 if $comment->status == $prev->status;

  1;
}
1;